#
# $Id: RenderableObject.py,v 1.10 2004/10/17 12:27:11 admin Exp $
#
# Copyright (c) 2003, 2004 Dirk Datzert
# All rights reserved. Written by Dirk Datzert <dirk@datzert.de>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__version__='$Revision: 1.10 $'[11:-2]

import string

from reportlab.pdfbase._fontdata import standardFonts

from reportlab.lib.attrmap import *
from reportlab.lib.colors import *
from reportlab.lib import validators

from reportlab.graphics.shapes import *
from reportlab.graphics.widgetbase import TypedPropertyCollection
from reportlab.graphics.charts.spider import SpiderChart
from reportlab.graphics.charts.textlabels import Label

from OFS.ObjectManager import ObjectManager
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import Item

from AccessControl.Role import RoleManager
from ComputedAttribute import ComputedAttribute

from Globals import DTMLFile, package_home
import Products

_dtmldir = os.path.join( package_home( globals() ), 'dtml' )


# ** 'bars','lines', 'slices' are not included **
# they need special handling, or the ChartTemplate.manage_test will not work

sub_attr = [ 'xValueAxis', 'yValueAxis', 'valueAxis', 'categoryAxis', 'lineLabels', 'labels', 'barLabels', 'legend1', 'strands' ]

boolean_attr = [ 'joinAxis', 'joinedLines', 'useAbsolute', 'useLines', 'useRects', 'debug' ]

string_attr = [ 'barLabelFormat', 'lineLabelFormat', 'labelTextFormat' ]

font_attr = [ 'fontName' ]


ADD_RENDERABLE_OBJECT_PERMISSION = 'Add RenderableObject'


def makeProperties(self, attr=None, klass=None):
	""" """

	if attr is not None:
		object_instance = attr
	elif klass in [Line, Rect, Ellipse]:
		object_instance = apply(klass, (0,0,0,0))
	elif klass == Circle:
		object_instance = apply(klass, (0,0,0))
	elif klass == String:
		object_instance = apply(klass, (0,0,'Text'))
	else:
		object_instance = apply(klass)

	_properties = ()

	if isinstance(object_instance, TypedPropertyCollection):
		object_instance = object_instance.__dict__['_value']

		p = ({ 'id': 'klass_collection', 'type': 'string', 'mode': '' },)
		_properties = p
		self.klass_collection = object_instance.__class__

	attr_map = object_instance._attrMap

	for attr_name in attr_map.keys():

		property_type = None
		select_variable = None
		attr = attr_map.data[attr_name]

		if isinstance(attr, AttrMapValue):	
			if isinstance(attr.validate, validators._isBoolean) \
				 or attr_name in boolean_attr:
				property_type = 'boolean'
			elif isinstance(attr.validate, validators._isNumber):
				property_type = 'float'
			elif isinstance(attr.validate, validators._isInt):
				property_type = 'int'
			elif isinstance(attr.validate, validators._isListOfStringsOrNone):
				property_type = 'lines'
			elif isinstance(attr.validate, validators.OneOf):
				property_type = 'selection'
				value = attr.validate._enum
				value = list(value)
				for i in range(len(value)):
					value[i] = str(value[i])
				value = tuple(value)
				select_variable = 'select_%s' % attr_name
				setattr (self, select_variable, value)
			elif isinstance(attr.validate, validators._isColor):
				property_type = 'selection'
				select_variable = 'possible_colors'
			elif isinstance(attr.validate, validators._isColorOrNone):
				property_type = 'selection'
				select_variable = 'possible_colors_or_none'
			elif attr_name in font_attr:
				property_type = 'selection'
				select_variable = 'possible_fonts'
			elif isinstance(attr.validate, validators._isString) \
				 or attr_name in string_attr:
				property_type = 'string'
			elif attr_name in sub_attr:
				attr = getattr(object_instance, attr_name)
				self._setObject(attr_name, RenderableObject(id=attr_name, attr=attr), set_owner=0)
				if attr_name == 'legend1':
					attr_name = '_legend2'
					attr = getattr(object_instance, attr_name)
					attr_name = 'legend2'
					self._setObject(attr_name, RenderableObject(id=attr_name, attr=attr), set_owner=0)
				continue

			if property_type is not None:
				if select_variable is not None:
					p = ({ 'id': attr_name, 'type': property_type, 
						  'select_variable': select_variable, 'mode': 'w' },)
				else:
					p = ({ 'id': attr_name, 'type': property_type, 'mode': 'w' },)

				value = getattr(object_instance, attr_name, None)

				if value == None:
					if property_type == 'boolean':
						value = 0
					elif property_type == 'string':
						value = ''
					elif property_type == 'lines' \
						 and attr_name == 'categoryNames':
						value = ['a','b','c','d']
					elif type == 'lines':
						value = []

				if value != None or select_variable is not None:
					if property_type == 'float':
						value = float(value)
					elif property_type == 'boolean':
						if value:
							value = 'on'

					_properties = _properties + p
					setattr(self, attr_name, value)

	return _properties


# from reportlab.lib.widgedbase.py

def _setRenderableProperties(self, propDict):
	"""Permits bulk setting of properties.	These may include
	child objects e.g. "chart.legend.width = 200".

	You can use e.g. "chart.bars[3].fillColor = HexColor(0xededed)"
	to set the 3rd fillColor for the 3rd bar row.

	All assignments will be validated by the object as if they
	were set individually in python code.

	All properties of a top-level object are guaranteed to be
	set before any of the children, which may be helpful to
	widget designers.
	"""

	childPropDicts = {}
	for (name, value) in propDict.items():
		parts = string.split(name, '.', 1)
		if len(parts) == 1:
			if self.__class__ == Label and name == 'text':
				value = string.replace(value, "\\n", "\n")
				self.setText(value)
			else:
				#simple attribute, set it now
				setattr(self, name, value)
		else:
			(childName, remains) = parts
			try:
				childPropDicts[childName][remains] = value
			except KeyError:
				childPropDicts[childName] = {remains: value}

	# now assign to children
	for (childName, childPropDict) in childPropDicts.items():
		parts = string.split(childName, '[', 1)

		if len(parts) == 1:

			# this is for 'chart.attribute = value'

			child = getattr(self, childName)
			
			if isinstance(child, TypedPropertyCollection):
				child.setProperties(childPropDict)
			else:
				_setRenderableProperties(child, childPropDict)
		else:

			# this is for 'chart.attribute[whatever] = value'

			(childName, remains) = parts

			child = getattr(self, childName)

			if isinstance(child, TypedPropertyCollection):
				idx = remains[:-1]
				parts = string.split(idx, ',', 1)

				if len(parts) == 1:

					# this is for 'chart.attribute[index] = value'

					idx = string.atoi(idx)
					child[idx].setProperties(childPropDict)

				else:

					# this is for 'chart.attribute[(row,col)] = value' or
					# 'chart.attribute[row,col] = value'

					row, col = parts

					# remove parentesis parts
					if row[0] == '(':
						row = row[1:]
						col = col[:-1]

					# get the int values
					row = string.atoi(row)
					col = string.atoi(col)

					# set the child value
					child[(row,col)].setProperties(childPropDict)


class RenderableObject(ObjectManager, PropertyManager, RoleManager, Item):
	"""
		Base class for all renderable graphics object
	"""

	meta_type = 'RenderableObject'

	_meta_types = ()

	_properties = ( { 'id' : 'klass', 'type' : 'string', 'mode' : '' }, )

	_manage_options = ObjectManager.manage_options \
		+ PropertyManager.manage_options \
		+ RoleManager.manage_options \
		+ Item.manage_options


	def _getManageOptions(self):
		""" """

		if len(self.objectIds()) == 0 \
			and len(self.all_meta_types()) == 0:
			return RenderableObject._manage_options[1:]
		else:
			return RenderableObject._manage_options


	manage_options = ComputedAttribute(_getManageOptions, 1)


	manage_propertiesForm = DTMLFile('dtml/properties', globals(),
                                   property_extensible_schema__=1)

	def __init__(self, id='', attr=None, klass=Drawing):
		""" init all """

		self.id = id

		if attr is not None:
			self.klass = attr.__class__
			self._properties = self._properties \
				+ makeProperties(self, attr=attr)
		else:
			self.klass = klass
			self._properties = self._properties \
				+ makeProperties(self, klass=self.klass)


	def all_meta_types(self):
		""" """

		f = lambda x,s=self: x['name'] in s._meta_types

		return filter ( f, Products.meta_types )


	def possible_colors_or_none(self):
		""" """

		return ['None',] + self.possible_colors()


	def possible_colors(self):
		""" """

		all_colors = getAllNamedColors().keys()
		all_colors.sort()
		return all_colors[3:] + all_colors[:3]


	def possible_fonts(self):
		""" """

		return standardFonts


	def createInstance(self, recursive=1):
		""" return a real instance """

		if self.hasProperty('klass_collection'):
			object_instance = TypedPropertyCollection(self.klass_collection)
		elif self.klass in [Line, Rect, Ellipse]:
			object_instance = apply (self.klass, (0,0,0,0))
		elif self.klass == Circle:
			object_instance = apply (self.klass, (0,0,0))
		elif self.klass == String:
			object_instance = apply (self.klass, (0,0,''))
		else:
			object_instance = apply (self.klass)

		if recursive:
			for sub_object in self.objectValues():
				if hasattr(object_instance, 'add'):
					object_instance.add(sub_object.createInstance(),
										 name=sub_object.id)
				else:
					attr_name = sub_object.id
					if attr_name == 'legend2':
						attr_name = '_legend2'
					setattr(object_instance, attr_name,
							 sub_object.createInstance())

		if isinstance(object_instance, TypedPropertyCollection):
			for (attr, value) in self.propertyItems():
				if attr in object_instance.getProperties().keys():
					property_type = self.getPropertyType(attr)

					if property_type == 'boolean':
						if hasattr(object_instance, attr):
							if type(value) == type(''):
								if value == 'on':
									value = 1
								else:
									value = 0
						else:
							value = 'None'

					if property_type == 'selection':
						try:
							value = toColor(value)
						except:
							pass

					if value != 'None':
						dict = {attr: value}
						object_instance.setProperties( dict )
			
		else:	
			for (attr, value) in self.propertyItems():
				if attr in object_instance._attrMap.keys():
					property_type = self.getPropertyType(attr)

					if property_type == 'boolean':
						if hasattr(object_instance, attr):
							if type(value) == type(''):
								if value == 'on':
									value = 1
								else:
									value = 0
						else:
							value = 'None'

					if property_type == 'string':
						if value == '':
							value = 'None'

					if property_type == 'selection':
						try:
							if attr.find('Color') >= 0:	
								if value != 'None':
									value = toColor(value)
								else:
									value = None

							if attr.find('joinAxisMode') >= 0:	
								if value == 'None':
									value = None
						except:
							pass

					if value != 'None':
						if object_instance.__class__ == Label \
							and attr == 'text':
							value = string.replace(value, "\\n", "\n")
							object_instance.setText(value)
						else:
							setattr(object_instance, attr, value)

		return object_instance

	manage_fixSelectionForm = \
		DTMLFile( 'manage_fixSelectionForm', _dtmldir )

	def manage_fixSelection(self, fixObject):
		""" fix select_joinAxisMode of all object in objectList """

		result = ''

		for path in fixObject:
			obj = self.unrestrictedTraverse(path)
			obj.fixSelection()
			result = result + path + " fixed\n"

		return result

	def fixSelection(self):
		""" fix select_joinAxisMode """

		if hasattr(self, 'select_joinAxisMode'):
			value = list(self.select_joinAxisMode)
			for i in range(len(value)):
				value[i] = str(value[i])
			setattr(self, 'select_joinAxisMode', tuple(value))


def manage_addRenderableObject(self, klass, id='', REQUEST=None):
	"""
	"""

	self = self.this()

	if id == '':
		self._object_counter = self._object_counter + 1
		id = '%s%s' % (klass.meta_type[3:], self._object_counter)

	n = apply( klass )
	n.id = id
	self._setObject( id, n )

	# return to the parent object's manage_main
	if REQUEST:
		url = REQUEST['URL1']
		REQUEST.RESPONSE.redirect( '%s/manage_main' % url )


#	vim:ts=4
